/**
 * 构建工作流
 */

const WebpackConfig = require('./webpack-config')
const { extendConfig, setMode, logVersion } = require('./common/util.js')
const logProgress = require('./common/log-progress')

module.exports = async () => {
  const mode = 'production'

  // 设置 webpack mode
  setMode(mode)

  // 打印 cli 版本号
  logVersion()

  // 扩展配置
  const extendConfigFunc = extendConfig()

  // 构建基础 webpack config
  let webpackConfig = WebpackConfig()

  // 执行扩展配置函数
  extendConfigFunc && await extendConfig({
    mode,
    webpackConfig
  })

  // 进度输出控制台
  logProgress.show(mode)

  // 启动 webpack
  webpackConfig = webpackConfig.toConfig ? webpackConfig.toConfig() : webpackConfig
  require('webpack')(webpackConfig, (err, stats) => {
    if (err) {
      console.error(err)
      return
    }

    console.log(stats.toString({
      chunks: false,
      colors: true
    }))
  })
}
